#!/usr/bin/python
# -*- coding: utf-8 -*-
#mmm
#KOPR 1.0
#mmm

from math import *
import alb3
import datetime

class ephem():

    def __init__(self, name='', tyear=0, tmonth=0, tday=0, q=0., e=0., peri=0., node=0., incl=0., eyear=0., emonth=0.,\
        eday=0., h0=0., n=0., ra='', de='', dist=0., r=0., elong=0., phasea=0., mag=0., az=0., alt=0., btime=0, tz = 0, tv = 0 ):
        self.name = name
        self.tyear, self.tmonth, self.tday = tyear, tmonth, tday # Perihelion date
        self.q, self.e = q, e
        self.peri, self.node, self.incl = peri, node, incl
        self.eyear, self.emonth, self.eday = eyear, emonth, eday # Epoch
        self.h0, self.n = h0, n # Magnitude parameters
        self.ra = ra
        self.de = de
        self.dist = dist
        self.r = r 
        self.elong = elong
        self.phase = phasea
        self.mag = mag
        self.az = az 
        self.alt = alt
        self.btime = btime
        self.tz = tz
        self.tv = tv
    
    def readdb(self, eline, h0 = 0, n = 0):
        rest, rest0, rest1, self.name = eline[14:81].split(), eline[81:90].strip(), eline[90:102].split() ,eline[102:159].strip() 
        self.tyear, self.tmonth, self.tday, self.q, self.e, self.peri, self.node, self.incl = int(rest[0]),int(rest[1]),float(rest[2]),\
         float(rest[3]),float(rest[4]), float(rest[5]), float(rest[6]), float(rest[7])

        if rest0:
            self.eyear, self.emonth, self.eday = float(rest0[0:4]), float(rest0[4:6]), float(rest0[6:8]) # Epoch

        if h0 == 0 and n == 0:
            self.h0, self.n = float(rest1[0]), float(rest1[1])# Magnitude parameters
        else:
            self.h0, self.n = h0, n
                                
    def compute(self, date, time, format = ['comet', 'extend', 'bestt'], longn = 0, lat = 0, timezone = 0, sunalt = 18, epoch = 2000.0 ):
        # Ephemeris parameters
        dyear,dmonth,dday = float(date.split('/')[0]), float(date.split('/')[1]), float(date.split('/')[2])
        dhour = float(time.split(':')[0]) + float(time.split(':')[1]) / 60.# - timezone 

        tdays = alb3.jday(self.tyear,self.tmonth,self.tday,0) - 2451543.5#2451545.0 
        ddays = alb3.jday(dyear,dmonth,dday,dhour) - 2451543.5#2451545.0 
        dt = ddays - tdays
        
        # 2.3. Calculations of Sun position
        meanlon, lonsun, rs, ecl = alb3.sunpos(ddays)
        xs, ys = alb3.suncoords(lonsun, rs)
        
        if 'comet' in format:
            #edays = alb3.jday(2000,1,1,12.) - 2451543.5 # positions for epoch 2000. else edays = ddays
            edays = ddays
            node = self.node - (3.82394E-5 * ( 365.2422 * ( epoch - 2000.0 ) - edays )) 

            # 2.1. Calculations object positions
            r, lonecl, latecl = alb3.truear(self.q, self.e, dt, self.peri, node, self.incl) # confirmed
            self.ra ,self.de, dist = alb3.objradec(r, lonecl, latecl, xs, ys, ecl) 
            self.r, self.dist, self.mag = r, dist, self.h0 + 5.* log10(dist) + 2.5 * self.n * log10(r) 
    
            if 'extend' in format: 
                lst, mst0 = alb3.lst(longn,alb3.jday(dyear,dmonth,dday,dhour))
                self.az ,self.alt = alb3.objaltaaz(lst, self.ra, self.de, lat)
                self.elong = degrees(acos( ( rs*rs + dist*dist - r*r ) / (2*rs*dist) ))
                self.phase = degrees(acos( ( r*r + dist*dist - rs*rs ) / (2*r*dist) ))

            if 'phasec' in format: 
                self.phasec, self.forwscac = alb3.phasec(rs,r,dist)
# forward scattering different d2g                self.forwscac = -alb3.fwscmag( degrees(acos( ( r*r + dist*dist - rs*rs ) / (2*r*dist) )) ,0.05,0.9,-0.6,0.95)
                        
            if 'bestt' in format:  
                gmst0 = degrees(lonsun) + 180.       
                ras, des = alb3.sunrade(xs, ys, ecl)                 
                lst, mst = alb3.lst(longn,ddays)
                self.tz, self.tv = alb3.riseset(lonsun, ras, des, longn, lat, sunalt )
                if self.tz != self.tv:
                  for i in range(4):
                    meanlon, lonsun, rs, ecl = alb3.sunpos(alb3.jday(dyear,dmonth,dday,self.tz) - 2451543.5 + 1.)
                    xs, ys = alb3.suncoords(lonsun, rs)
                    ras, des = alb3.sunrade(xs, ys, ecl) 
                    self.tz = alb3.riseset(lonsun, ras, des, longn, lat, sunalt )[0]
                  for i in range(4):
                    meanlon, lonsun, rs, ecl = alb3.sunpos(alb3.jday(dyear,dmonth,dday,self.tv) - 2451543.5)
                    xs, ys = alb3.suncoords(lonsun, rs)
                    ras, des = alb3.sunrade(xs, ys, ecl) 
                    self.tv = alb3.riseset(lonsun, ras, des, longn, lat, sunalt )[1]
                self.btime = alb3.transit(gmst0, longn, self.ra, self.tz, self.tv )

                ddays = alb3.jday(dyear,dmonth,dday,self.btime) - 2451543.5
                dt = ddays - tdays
                edays = ddays
                node = self.node - (3.82394E-5 * ( 365.2422 * ( epoch - 2000.0 ) - edays )) 
                r, lonecl, latecl = alb3.truear(self.q, self.e, dt, self.peri, node, self.incl)
                self.ra ,self.de, dist = alb3.objradec(r, lonecl, latecl, xs, ys, ecl) 
                self.r, self.dist, self.mag = r, dist, self.h0 + 5.* log10(dist) + 2.5 * self.n * log10(r) 

def calcSun(date, time = '00:00',longn=0,lat=0):
    dyear,dmonth,dday = float(date.split('/')[0]), float(date.split('/')[1]), float(date.split('/')[2])
    dhour = float(time.split(':')[0]) + float(time.split(':')[1]) / 60.# - timezone 

    ddays = alb3.jday(dyear,dmonth,dday,dhour) - 2451543.5

    L, lon, r, ecl = alb3.sunpos(ddays) # fce sun
    ra, de = alb3.sunradec(r, lon, ecl)
        
    lst, mst0 = alb3.lst(longn,alb3.jday(dyear,dmonth,dday,dhour))  
    az, alt = alb3.objaltaaz(lst, ra, de, lat)
    
    return az, alt

def calcMoon(date, time = '00:00',longn=0,lat=0):
    dyear,dmonth,dday = float(date.split('/')[0]), float(date.split('/')[1]), float(date.split('/')[2])
    dhour = float(time.split(':')[0]) + float(time.split(':')[1]) / 60.# - timezone 
    ddays = alb3.jday(dyear,dmonth,dday,dhour) - 2451543.5

    meanlon, sollon, rs, ecl = alb3.sunpos(ddays) # fce sun
    lonecl, latecl, ra, de, r = alb3.moonpos(ddays, ecl) # fce moon

    lst, mst0 = alb3.lst(longn,alb3.jday(dyear,dmonth,dday,dhour))
    az, alt = alb3.objaltaaz(lst, ra, de, lat)
    mpar = degrees(asin( 1./r ))
    alt = alt - mpar * cos(radians(alt))  
    elong = acos( cos(sollon - lonecl) * cos(latecl) )
    fv = pi - elong
    phase  =  ( 1. + cos(fv) ) / 2.

    return az, alt, phase    

def StarAlt(ra, de, date, time = '00:00',longn=0,lat=0): # ra in hours, de in degrees, longn + lat degrees
    dyear,dmonth,dday = float(date.split('/')[0]), float(date.split('/')[1]), float(date.split('/')[2])
    dhour = float(time.split(':')[0]) + float(time.split(':')[1]) / 60. 
    jday = alb3.jday(dyear,dmonth,dday,dhour)
    
    ra, de = alb3.appradec(ra,de,jday)
    lst, mst0 = alb3.lst(longn,jday)  
    az, alt = alb3.objaltaaz(lst, ra, de, lat)

    return alt

def j2000rade(ra, de, date, time = '00:00'): # ra in hours, de in degrees, longn + lat degrees
    dyear,dmonth,dday = float(date.split('/')[0]), float(date.split('/')[1]), float(date.split('/')[2])
    dhour = float(time.split(':')[0]) + float(time.split(':')[1]) / 60. 
    jday = alb3.jday(dyear,dmonth,dday,dhour)

    ra, de = alb3.normradec(ra,de,jday)

    return ra, de

def apprade(ra, de, date, time = '00:00'): # ra in hours, de in degrees, longn + lat degrees
    dyear,dmonth,dday = float(date.split('/')[0]), float(date.split('/')[1]), float(date.split('/')[2])
    dhour = float(time.split(':')[0]) + float(time.split(':')[1]) / 60. 
    jday = alb3.jday(dyear,dmonth,dday,dhour)
       
    ra, de = alb3.appradec(ra,de,jday)

    return ra, de

def eDatetoJday(date):
    try:    
      y, m, d = date.split('/')
    except ValueError:
      print(('Error in input value: ', date))
      
    y, m, d = float(y), float(m), float(d)

    return alb3.jday( y,m,d,0 )

def JdaytoeDate(jd):
    year, month, day = alb3.gdate(jd)
    return '{:d}/{:0>2.0f}/{:0>5.2f}'.format( year, float(month), day )        
    
def JdaytoDtime(jd):
    year, month, day = alb3.gdate(jd)
    h = modf(day)[0]*24.
    m = modf(h)[0]*60.
    s = modf(m)[0]*60.
    return datetime.datetime(int(year), int(month), int(day), int(h), int(m), int(s) )     

def DatetoDatetime(year, month, day):
    h = modf(day)[0]*24.
    m = modf(h)[0]*60.
    s = modf(m)[0]*60.
    return datetime.datetime(int(year), int(month), int(day), int(h), int(m), int(s) )     
