#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
# Common dialogs
#"""

import os
from math import *
import datetime
import comephem
import koprfunc
import koprhand
import koprclass
import koprfig
import urllib.request, urllib.error, urllib.parse
import datavar
import ssl
from PyQt5 import QtCore, QtGui, QtWidgets
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
import matplotlib.pyplot as plt

font = datavar.DefaultFont()

ctx = ssl.create_default_context()
ctx.check_hostname = False
ctx.verify_mode = ssl.CERT_NONE

def InfoDialog(typ, text, message, errormsg = '', errordetmsg = ''):
    msg = QtWidgets.QMessageBox()

    if typ == 1:
        msg.setIcon(QtWidgets.QMessageBox.Critical)
    elif typ == 2:
        msg.setIcon(QtWidgets.QMessageBox.Warning)
    else:
        msg.setIcon(QtWidgets.QMessageBox.Information)

    msg.setWindowTitle(text)
    msg.setText(message)
    msg.setStandardButtons(QtWidgets.QMessageBox.Ok)
    msg.setInformativeText(errormsg)
    msg.setDetailedText(errordetmsg)
    retval = msg.exec_()

class AddComet(QtWidgets.QDialog):
    def __init__(self, parent = None):
        super(AddComet, self).__init__()

        vbox = QtWidgets.QVBoxLayout()

        comlbl = QtWidgets.QLabel('Insert Comet (e.g. 103P or C/2012 X1): ', self)# line 1
        comlbl.setFont(font)
        self.lecom = QtWidgets.QLineEdit(self)
        self.lecom.setFont(font)
        self.btnmpc = QtWidgets.QPushButton('Load from MPC', self)
        self.btnmpc.setFont(font)
        self.btnmpc.clicked.connect(self.button_load_comet)
        hbox = QtWidgets.QHBoxLayout()
        hbox.addWidget(comlbl)
        hbox.addWidget(self.lecom)
        hbox.addWidget(self.btnmpc)
        vbox.addLayout(hbox)

        namelbl = QtWidgets.QLabel('Full name: ', self)# line 1
        namelbl.setFont(font)
        self.lename = QtWidgets.QLineEdit(self)
        self.lename.setFont(font)
        self.cbcom = QtWidgets.QComboBox(self)
        self.cbcom.setStyleSheet("selection-color: black")
        kinofile = open('./config-files/CustomElements.dat', 'r')
        self.cbcom.addItem('Select comet')
        for line in kinofile.readlines():
            self.cbcom.addItem(line[102:159].strip())
        kinofile.close()
        self.cbcom.setFont(font)
        self.cbcom.activated[str].connect(self.onActivated)
        hbox = QtWidgets.QHBoxLayout()
        hbox.addWidget(namelbl)
        hbox.addWidget(self.lename)
        hbox.addWidget(self.cbcom)
        vbox.addLayout(hbox)

        grid = QtWidgets.QGridLayout()
        self.input, names, title = {}, [ 'Perihelion date: ', 'Epoch: ', 'q: ', 'Peri: ', 'e: ', 'Node: ', '', 'Incl.: ', 'H0: ', 'n (slope): ' ],\
                                                  'Custom Comtes'

        for i in range(5):
            label = QtWidgets.QLabel(names[i*2])
            label.setFont(font)
            grid.addWidget(label, *(i,0))
            label = QtWidgets.QLabel(names[i*2+1])
            label.setFont(font)
            grid.addWidget(label, *(i,2))

            self.input[i*2] = QtWidgets.QLineEdit(self)
            self.input[i*2].setFont(font)
            grid.addWidget(self.input[i*2], *(i,1))
            self.input[i*2+1] = QtWidgets.QLineEdit(self)
            self.input[i*2+1].setFont(font)
            grid.addWidget(self.input[i*2+1], *(i,3))
        vbox.addLayout(grid)
        self.input[8].setText('10.0')
        self.input[9].setText('4.0')

        self.btnsave = QtWidgets.QPushButton('Save custom elements', self)
        self.btnsave.setFont(font)
        self.btnsave.clicked.connect(self.button_save_comet)
        self.btndel = QtWidgets.QPushButton('Delete custom elements', self)
        self.btndel.setFont(font)
        self.btndel.clicked.connect(self.button_delete_comet)
        btncanc = QtWidgets.QPushButton('Cancel', self)
        btncanc.setFont(font)
        btncanc.clicked.connect(self.button_click_cancel)
        hbox = QtWidgets.QHBoxLayout()
        hbox.addWidget(self.btnsave)
        hbox.addWidget(self.btndel)
        hbox.addWidget(btncanc)
        vbox.addLayout(hbox)

        self.setLayout(vbox)  
        self.setWindowTitle(title)
        self.setWindowModality(QtCore.Qt.ApplicationModal)
        self.exec_()

    def button_click_cancel(self):
        self.close()
            
    def onActivated(self, value):
        if value == 'Select comet':
            self.button_clear()
        else:
            kinofile = open('./config-files/CustomElements.dat', 'r')
            output = kinofile.readlines()
            kinofile.close()
    
            comet = comephem.ephem()
            
            tdate, name, peri, node, incl, e, q ,epoch, h0, slope = '','','','','','','','','','' 
            for line in output:
                if line[102:159].startswith(str(value)):
                    comet.readdb(line)                    
                    tdate = str(comet.tyear)  + '/' + str(comet.tmonth) + '/' + str(comet.tday)  
                    name, peri, node, incl = comet.name, str(comet.peri), str(comet.node), str(comet.incl)
                    e, q, h0, slope = str(comet.e), str(comet.q), str(comet.h0), str(comet.n) 
                    epoch = str(int(comet.eyear))  + '/' + str(int(comet.emonth)) + '/' + str(comet.eday)
            self.lecom.clear()
            self.lename.setText(name)
            self.input[0].setText(tdate)
            self.input[1].setText(epoch)
            self.input[2].setText(q)
            self.input[3].setText(peri)
            self.input[4].setText(e)
            self.input[5].setText(node)
            self.input[7].setText(incl)
            self.input[8].setText(h0)
            self.input[9].setText(slope)  

    def button_load_comet(self):
        destig = str(self.lecom.text())
        link = 'https://www.minorplanetcenter.net/db_search/show_object?utf8=%E2%9C%93&object_id='+destig.replace('/','%2F').replace(' ','+')
        response = urllib.request.urlopen(link, context=ctx)
        print(link)

        cname = response.read().decode('UTF-8')
        name = cname.split('<h3>',1)[1].split('</h3>',1)[0].replace('\n','')
        name = ' '.join(name.split())
        
        response = urllib.request.urlopen('http://www.minorplanetcenter.net/db_search/show_object?utf8=%E2%9C%93&object_id='+destig.replace('/','%2F').replace(' ','+'), context=ctx)
        html = response.readlines()

        done = 0
        for line in html:
            line = line.decode('UTF-8')
            if '<td>epoch</td>' in line and done<10:
                epoch = line.strip().split('<td class="rj">')[1].replace('</td></tr>','')
                done = done +1
            elif '<td>perihelion date</td>' in line and done<10:
                tdate = line.strip().split('<td class="rj">')[1].replace('</td></tr>','')
                done = done +1
            elif '<td>argument of perihelion (' in line and done<10:
                peri = line.strip().split('<td class="rj">')[1].replace('</td></tr>','')
                done = done +1
            elif '<td>ascending node (' in line and done<10:
                node = line.strip().split('<td class="rj">')[1].replace('</td></tr>','')
                done = done +1
            elif '<td>inclination (' in line and done<10:
                incl = line.strip().split('<td class="rj">')[1].replace('</td></tr>','')
                done = done +1
            elif '<td>eccentricity</td>' in line and done<10:
                e = line.strip().split('<td class="rj">')[1].replace('</td></tr>','')
                done = done +1
            elif '<td>perihelion distance (AU)</td>' in line and done<10:
                q = line.strip().split('<td class="rj">')[1].replace('</td></tr>','')
                done = done +1

        self.lename.setText(name)
        try:
          self.input[0].setText(tdate.replace('-','/'))
          self.input[1].setText(epoch.replace('-','/'))
          self.input[2].setText(q)
          self.input[3].setText(peri)
          self.input[4].setText(e)
          self.input[5].setText(node)
          self.input[7].setText(incl)
        
          kinofile = open('./config-files/CustomElements.dat', 'r')
          for line in kinofile.readlines():
            if line.startswith(destig):
              InfoDialog(0, 'Success', destig + 'is already in custom comets database')
          kinofile.close()
        except:
          InfoDialog(1, 'Unknown Comet', 'Check comet destignation.' )

    def button_save_comet(self):
        elements = [ str(self.lename.text()), str(self.input[0].text()), str(self.input[1].text()), float(self.input[4].text()), float(self.input[2].text()), float(self.input[7].text()),\
                     float(self.input[5].text()), float(self.input[3].text()), float(self.input[8].text()), float(self.input[9].text()) ]

        if '' in elements:
            InfoDialog(1, 'Error', 'Fill all required fields!')
        else:
            name,tdate,epoch,e,q,incl,node,peri,h0,slope = elements
            year,month,day = int(tdate.split('/')[0]),int(tdate.split('/')[1]),float(tdate.split('/')[2])
            outputdatet = '{:4d} {:02d} {:07.4f}'.format(year,month,day)
            eyear,emonth,eday = int(epoch.split('/')[0]),int(epoch.split('/')[1]),int(float(epoch.split('/')[2]))
            epoch = '{:4d}{:02d}{:02d}'.format(eyear,emonth,eday) 
            elements = '              {:15s} {:10.7f} {:9.7f} {:9.5f} {:9.5f} {:9.5f} {:8s}  {:4.1f} {:4.1f}  {:s}'\
                       .format( outputdatet,q,e,peri,node,incl,epoch,h0,slope,name )

            kinofile = open('./config-files/CustomElements.dat', 'r')
            filelines = kinofile.readlines()
            kinofile.close()

            kinofile = open('./config-files/CustomElements.dat', 'w')
            for line in filelines:
              if line[102:159].strip() != name:
                kinofile.write(line)

            kinofile.write(elements+'\n')
            
            InfoDialog(0, 'Success', 'Comet '+name+' was added')
            
            self.cbcom.clear()
            kinofile = open('./config-files/CustomElements.dat', 'r')
            self.cbcom.addItem('Select comet')
            for line in kinofile.readlines():
                self.cbcom.addItem(line[102:159].strip())
            kinofile.close()
    
    def button_delete_comet(self):
        destig = str(self.lename.text())
        
        kinofile = open('./config-files/CustomElements.dat', 'r')
        output = kinofile.readlines()
        kinofile.close()
        
        kinofile = open('./config-files/CustomElements.dat', 'w')
        for line in output:
            if not line[102:159].startswith(destig):
                kinofile.write(line)
            else:
                InfoDialog(0, 'Success', 'Comet '+destig+' was removed')
        kinofile.close()

        self.cbcom.clear()
        kinofile = open('./config-files/CustomElements.dat', 'r')
        self.cbcom.addItem('Select comet')
        for line in kinofile.readlines():
            self.cbcom.addItem(line[102:159].strip())
        kinofile.close()
        
        self.button_clear()


# ************** Common dialogs wvisual and wccd observations ***************************

# --- list of reference stars ---
class StarDialog(QtWidgets.QDialog):
    def __init__(self, parent = None, typ = '', refno = 0, destig = '', date = '', time = '', extinct = '' ,lon = '', lat = '', elev = '', stars = '' ):
        super(StarDialog, self).__init__()
        self.typ, self.refno, self.destig, self.date, self.time, self.extinct, self.lon, self.lat, self.elev, self.stars, self.output =\
        typ, refno, destig, date, time, extinct, lon, lat, elev, stars, stars
        if len(self.stars) > 0 and self.stars[0] != '':
            self.refno = len(self.stars)
            if self.typ == 'ccd':
                avgstar = koprfunc.MeanStar(self.stars)
                for star in self.stars:
                    if 'c' in star[0]:
                        self.refno = self.refno - 1

        vbox = QtWidgets.QVBoxLayout()

        maglbl = QtWidgets.QLabel('Magnitude source: ', self)
        self.comsrc = QtWidgets.QLineEdit(self)
        try:
          self.comsrc.setText(self.stars[0][6])
        except:
          print('no stars')
          
        self.comsrc.setDisabled(1)
        extlbl = QtWidgets.QLabel('Extinction', self)
        self.comext = QtWidgets.QComboBox(self)
        self.comext.setStyleSheet("selection-color: black")
#        self.comext.activated[str].connect(self.onActivatedext)
        for extinct in datavar.Extinct():  
            self.comext.addItem(extinct)
        
        self.comext.setCurrentIndex(self.extinct)
        
        elements = [ maglbl, self.comsrc, extlbl, self.comext ]
        grid = QtWidgets.QGridLayout()
        for i in range(len(elements)):
          elements[i].setFont(font)
          grid.addWidget(elements[i], *(0, i) )
       
        vbox.addLayout(grid)
        
        hLine = QtWidgets.QFrame()
        hLine.setFrameStyle(QtWidgets.QFrame.HLine)
        
        vbox.addWidget(hLine)
        
        grid = QtWidgets.QGridLayout()
        self.starlbl, self.asle, self.magle, self.bmvle, self.altle, self.catle, self.devi = {}, {}, {}, {}, {}, {}, {}
        for i in range( self.refno ):
            self.starlbl[i] = QtWidgets.QCheckBox('Star nr.'+str(i+1), self)
            self.starlbl[i].toggle()
            self.starlbl[i].setFont(font)
            grid.addWidget(self.starlbl[i], *(i+1, 0) )        

            self.asle[i] = QtWidgets.QLineEdit(self)
            self.asle[i].setFont(font)
            self.asle[i].setFixedWidth(100)
            grid.addWidget(self.asle[i], *(i+1, 1) )        

            self.magle[i] = QtWidgets.QLineEdit(self)
            self.magle[i].setFont(font)
            self.magle[i].setFixedWidth(100)
            grid.addWidget(self.magle[i], *(i+1, 2) )        

            self.bmvle[i] = QtWidgets.QLineEdit(self)
            self.bmvle[i].setFont(font)
#            self.bmvle[i].setDisabled(1)
            self.bmvle[i].setFixedWidth(100)
            grid.addWidget(self.bmvle[i], *(i+1, 3) )        

            self.altle[i] = QtWidgets.QLineEdit(self)
            self.altle[i].setFont(font)
#            self.altle[i].setDisabled(1)
            self.altle[i].setFixedWidth(100)
            grid.addWidget(self.altle[i], *(i+1, 4) )        

            if self.typ == 'ccd':
              self.devi[i] = QtWidgets.QLineEdit(self)
              self.devi[i].setFont(font)
              self.devi[i].setFixedWidth(100)              
              self.devi[i].setDisabled(1)
              grid.addWidget(self.devi[i], *(i+1, 5) )

        names, ran = [ ' Star nr.', ' Argelander deg.', ' Star magnitude', ' B-V mag', ' Altitude' ], 5
        if self.typ == 'ccd':
            names, ran = [ ' Star nr.', ' Relative mag.', ' Star magnitude', ' B-V mag', ' Altitude', 'Deviation' ], 6
            self.starlbl[self.refno] = QtWidgets.QLabel('Comet')
            self.starlbl[self.refno].setFont(font)
            grid.addWidget(self.starlbl[self.refno], *(self.refno+1, 0) )        

            self.asle[self.refno] = QtWidgets.QLineEdit(self)
            self.asle[self.refno].setFont(font)
            self.asle[self.refno].setFixedWidth(100)
            grid.addWidget(self.asle[self.refno], *(self.refno+1, 1) )        

            self.altle[self.refno] = QtWidgets.QLineEdit(self)
            self.altle[self.refno].setFont(font)
            self.altle[self.refno].setDisabled(1)
            self.altle[self.refno].setFixedWidth(100)
            grid.addWidget(self.altle[self.refno], *(self.refno+1, 4) )        

        positions = [(0,j) for j in range(ran)]
        for position, name in zip(positions, names):
            label = QtWidgets.QLabel(name)
            label.setFont(font)
            grid.addWidget(label, *position)

        if self.typ == 'ccd':
            btndev = QtWidgets.QPushButton('Update deviations', self)
            btndev.setFont(font)
            btndev.clicked.connect(self.button_click_devupd)
            for star in self.stars: 
                if star[0] == 'c':
                    self.asle[self.refno].setText(str(star[1]))
                    self.altle[self.refno].setText(str(star[4]))

        if len(self.stars) > 0 and self.stars[0] != '':
            count = 0
            for star in self.stars:
                if star[0] != 'c':
                    if star[0] == 'x':
                        self.starlbl[count].setChecked(0)
                    self.asle[count].setText(str(star[1]))
                    self.magle[count].setText(str(star[2]))
                    self.bmvle[count].setText(str(star[3]))
                    self.altle[count].setText(str(star[4]))
                    if self.typ == 'ccd' and star[0] == 'o': 
                      self.devi[count].setText('{:4.2f} mag'.format(round(avgstar - (star[1] - star[2]),2)))
                    count += 1
                   
        vbox.addLayout(grid)
        
        hLine = QtWidgets.QFrame()
        hLine.setFrameStyle(QtWidgets.QFrame.HLine)
        vbox.addWidget(hLine)

        btnclr = QtWidgets.QPushButton('Clear all stars', self)
        btnclr.setFont(font)
        btnclr.clicked.connect(self.button_click_clear)

        btnstar = QtWidgets.QPushButton('Save and quit', self)
        btnstar.setFont(font)
        btnstar.clicked.connect(self.button_click_insert)
        
        btncanc = QtWidgets.QPushButton('Cancel', self)
        btncanc.setFont(font)
        btncanc.clicked.connect(self.button_click_cancel)
        
        hbox = QtWidgets.QHBoxLayout()
        hbox.addWidget(btnclr)
        if self.typ == 'ccd':
          hbox.addWidget(btndev)
        hbox.addWidget(btnstar)
        hbox.addWidget(btncanc)
        vbox.addLayout(hbox)
        
        self.setLayout(vbox)  
        self.setWindowTitle("Insert stars")
        self.setWindowModality(QtCore.Qt.ApplicationModal)

    def button_click_clear(self):    
        self.output, self.extinct = [] , self.comext.currentIndex()
        self.close()

    def button_click_cancel(self):    
        self.output = self.stars
        self.close()

    def button_click_devupd(self):
        output = []
        for i in range( self.refno ):
          if self.starlbl[i].isChecked():
            output.append(['o', float(self.asle[i].text()), float(self.magle[i].text())])
        avgstar = koprfunc.MeanStar(output)
        for i in range( self.refno ):
          if self.starlbl[i].isChecked():
            self.devi[i].setText('{:4.2f} mag'.format(round(avgstar - (float(self.asle[i].text()) - float(self.magle[i].text())),2)))
          else:
            self.devi[i].clear()
                
    def button_click_insert(self):
        critical,errcount,errormsg,errordetmsg = 0, 0, '', ''
        output = []
        
        if self.typ == 'ccd' and self.asle[self.refno].text() != '':
            output.append(['c', float(self.asle[self.refno].text()), 0, '', str(self.altle[self.refno].text()), '', str(self.comsrc.text())])

        for i in range( self.refno ):
            if self.starlbl[i].isChecked() and str(self.comext.currentText()) == 'No Extinction':
                if self.asle[i].text() and self.magle[i].text():
                    output.append(['o', float(self.asle[i].text()), float(self.magle[i].text()), str(self.bmvle[i].text()), str(self.altle[i].text()), '', str(self.comsrc.text())] )
                else:
                    critical, errormsg, errordetmsg = 1, 'Input error\n', '* Required value (Argelander degree, Magnitude) is missing for one or more stars\n'
                    errcount += 1 
            elif self.starlbl[i].isChecked() and str(self.comext.currentText()) != 'No Extinction':
                if self.asle[i].text() and self.magle[i].text() and self.altle[i].text():
                    output.append(['o', float(self.asle[i].text()), float(self.magle[i].text()), str(self.bmvle[i].text()), str(self.altle[i].text()), '', str(self.comsrc.text())] )        
                else:
                    critical, errormsg, errordetmsg = 1, 'Input error\n', '* Required value (Argelander degree, Magnitude, Altitude) is missing for one or more stars\n'
                    errcount += 1 
            else:
                output.append( ['x', float(self.asle[i].text()), float(self.magle[i].text()), str(self.bmvle[i].text()), str(self.altle[i].text()), '', str(self.comsrc.text())] )

        if critical == 1:
            InfoDialog(1, 'Errors were encountered', str(errcount) + ' errors occured: ', errormsg, errordetmsg)
        else:
            self.output, self.extinct = output, self.comext.currentIndex()
            self.close()
        
# --- Output of observation        
class ObsOutput(QtWidgets.QDialog):
    def __init__(self, output1, output2, obscode, plot = [], parent = None, afrho = ''):
        super(ObsOutput, self).__init__()
        self.output, self.output2, self.obscode, self.title = output1, output2, obscode, plot[5] 
        vbox = QtWidgets.QVBoxLayout()
        
        output = QtWidgets.QTextEdit()
        if afrho:

          #formatting afrho values START
          afr_fmt = []
          for line in afrho:
              item = line.split(',')
              newline = []
              newline.append('%14s' % item[0])
              newline.append('%15s' % item[1])
              newline.append('%7s' % item[2])
              newline.append('%7s' % item[3])
              newline.append('%7s' % item[4])
              newline.append('%4s' % item[5])
              newline.append('%10s' % item[6])
              newline.append('%9s' % item[7])
              newline.append('%7s' % item[8])
              afr_fmt.append(','.join(newline))
          #formatting afrho values END
            
          output.setText('ICQ Code:\n'+output1+'\nCometObs format:\n'+output2+'\n\nAfrho output:\n'+'|OBJECT       |    JULIAN DATE| R (AU)| D (AU)|PH. ANG|BAND| RADIUS KM|    AFRHO|  ERROR|\n'+'\n'.join(afr_fmt))
          self.sc = koprfig.VisObsCanvas(self, xlist=plot[0], ylist=plot[1], t=plot[2], s=plot[3], mag=plot[4], title=self.title, afrad=plot[6], afrho=plot[7] )
        else:
          self.sc = koprfig.VisObsCanvas(self, xlist=plot[0], ylist=plot[1], t=plot[2], s=plot[3], mag=plot[4], title=self.title )
          output.setText('ICQ Code:\n'+output1+'\nCometObs format:\n'+output2)
        output.setFont(font)
        output.setReadOnly(1)
        output.setMinimumSize(600,160)
        vbox.addWidget(output)

        vbox.addWidget(self.sc)

        btncomp = QtWidgets.QPushButton('COBS data comparison', self)
        btncomp.setFont(font)
        btncomp.clicked.connect(self.button_click_comp)

        btnsfig = QtWidgets.QPushButton('Save figure', self)
        btnsfig.setFont(font)
        btnsfig.clicked.connect(self.button_click_sfig)

        btn = QtWidgets.QPushButton('Append ICQ code', self)
        btn.setFont(font)
        btn.clicked.connect(self.button_click_save)

        btncanc = QtWidgets.QPushButton('Close', self)
        btncanc.setFont(font)
        btncanc.clicked.connect(self.button_click_cancel)

        hbox = QtWidgets.QHBoxLayout()
        hbox.addWidget(btncomp)
        hbox.addWidget(btnsfig)
        hbox.addWidget(btn)
        hbox.addWidget(btncanc)

        vbox.addLayout(hbox)
        
        self.setLayout(vbox)  
        self.setWindowTitle('Observation output '+self.title)
        self.setWindowModality(QtCore.Qt.ApplicationModal)

    def button_click_cancel(self):
        self.close()

    def button_click_save(self):
        obs_file = open(self.obscode+'.txt', 'a')
        obs_file.write(self.output)
        obs_file.close()

    def button_click_comp(self):
        destig = self.output2.split('\n',1)[0].replace(':','')
        if not destig[0:1].isdigit():
            destig = destig.split()[0]+' '+destig.split()[1]
        else:
            destig = destig.split('/')[0]
        a = COBSComp(destig, self.output)
        a.exec_()
        
    def button_click_sfig(self):
        title = self.title.replace(' ','_').replace('/','-').replace(':','-')
        fname = QtWidgets.QFileDialog.getSaveFileName( None, 'Save file', os.getcwd()+'/'+title+'.png', filter ='png (*.png)')
        if fname != '':
            self.sc.save_figure(str(fname[0]))    

# ************** END ***************************

# ************** Dialogs wcomparison  ***************************
class CometSelector(QtWidgets.QDialog):
    def __init__(self, script, parent = None):
        super(CometSelector, self).__init__()
        self.output = ''
        vbox = QtWidgets.QVBoxLayout()

        examplelines = ['Add comets, each line for comet, values (separator ;) : ', 
                        'Destignation, Group number, Types (v = visual c = ccd m = mpc total, n = mpc nuclear), Returns (separator /), Formulas (separator /)' , 
                        'Example: ' , 
                        '103P;0;vcmn;2010/2023;H0=9.46  mag, n=10.15 [-232.9, 0.0]/H0=8.71  mag, n=5.05  [0.0, 472.6]' ]

        label = QtWidgets.QLabel('\n'.join(examplelines) )
        label.setFont(font)
        vbox.addWidget(label)

        self.field = QtWidgets.QTextEdit('')
        self.field.setText( '\n'.join(script) )
        self.field.setFont(font)
        self.field.setMinimumSize(600,400)
        vbox.addWidget(self.field)

        btnadd = QtWidgets.QPushButton('Add from list', self)
        btnadd.setFont(font)
        btnadd.clicked.connect(self.button_click_add)

        btnload = QtWidgets.QPushButton('Load Script', self)
        btnload.setFont(font)
        btnload.clicked.connect(self.button_click_load)

        btnsave = QtWidgets.QPushButton('Save Script', self)
        btnsave.setFont(font)
        btnsave.clicked.connect(self.button_click_save)

        btngo = QtWidgets.QPushButton('Run Script', self)
        btngo.setFont(font)
        btngo.clicked.connect(self.button_click_go)

        hbox = QtWidgets.QHBoxLayout()
        
        hbox.addWidget(btnadd)
        hbox.addWidget(btnload)
        hbox.addWidget(btnsave)
        hbox.addWidget(btngo)
        
        vbox.addLayout(hbox)

        self.setLayout(vbox)  
        self.setWindowTitle('Comet comparison script')
        self.setWindowModality(QtCore.Qt.ApplicationModal)

    def button_click_add(self):
        text, ok = QtWidgets.QInputDialog.getText(self, 'Select comets', 'Input Comets (e.g. 103P or C/2012 X1) separated by commas, none for all: ')  
        if text:
          destigs = text.split(',')
          for item in destigs:
            item = item.strip()
            newline = []
            newline.append(item)
            newline.append(str(len(str(self.field.toPlainText()).split('\n'))))
            newline.append('vcmn')
            newline.append('')
            kometa = koprclass.comet()
            kometa.LoadComet(item)
            magparams = []
            for line in kometa.magparams:
              destig, h0, n, tbeg, tend = line.split(',')
              magparams.append( 'H0={:5s} mag, n={:5s} [{:s}, {:s}]'.format(h0, n, tbeg, tend.replace('\n','')) )
            if not(magparams):
              InfoDialog(1, 'Comet error', 'Elements not found! Add this comet first.')
            else:
              newline.append( '/'.join( magparams) )
              outline = ';'.join( newline )
              actual = str(self.field.toPlainText()).split('\n')
              actual.append(outline)
              self.field.setText( '\n'.join(actual) )

    def button_click_load(self):
        fname = QtWidgets.QFileDialog.getOpenFileName( None, 'Open file', './' )
        if fname != '':
            obs_file = open(fname[0], 'r')
            content = obs_file.read()
            self.field.setText( content )
            obs_file.close()

    def button_click_save(self):
        fname = QtWidgets.QFileDialog.getSaveFileName( None, 'Save file', './' )
        if fname != '':
            obs_file = open(fname[0], 'w')
            obs_file.write(str(self.field.toPlainText()))
            obs_file.close()

    def button_click_go(self):
        self.output = []
        for item in str(self.field.toPlainText()).split('\n'):
          if item != '':
            self.output.append(item)
        self.close()

# ************** END ***************************

# ************** Common dialogs wccd and wanalyzer  ***************************
class MagDataOutput(QtWidgets.QDialog):
    def __init__(self, magdata, source, parent = None):
        super(MagDataOutput, self).__init__()
        self.output = magdata
        vbox = QtWidgets.QVBoxLayout()

        label = QtWidgets.QLabel(source)
        label.setFont(font)
        vbox.addWidget(label)

        output = QtWidgets.QTextEdit()
        output.setText(magdata)
        output.setFont(font)
        output.setReadOnly(1)
        output.setMinimumSize(600,600)
        output.setLineWrapMode(QtWidgets.QTextEdit.NoWrap)
        vbox.addWidget(output)

        btncanc = QtWidgets.QPushButton('Close', self)
        btncanc.setFont(font)
        btncanc.clicked.connect(self.button_click_cancel)

        vbox.addWidget(btncanc)

        self.setLayout(vbox)  
        self.setWindowTitle(source)
        self.setWindowModality(QtCore.Qt.ApplicationModal)

    def button_click_cancel(self):
        self.close()

# ************** END ***************************

class ICQData(QtWidgets.QDialog):
    def __init__(self, magdata, source, parent = None):
        super(ICQData, self).__init__()
        self.output = magdata
        vbox = QtWidgets.QVBoxLayout()

        label = QtWidgets.QLabel(source)
        label.setFont(font)
        vbox.addWidget(label)

        self.textfield = QtWidgets.QTextEdit()
        self.textfield.setText(magdata)
        self.textfield.setFont(font)
#        output.setReadOnly(1)
        self.textfield.setMinimumSize(600,600)
        self.textfield.setLineWrapMode(QtWidgets.QTextEdit.NoWrap)
        vbox.addWidget(self.textfield)

        btnsave = QtWidgets.QPushButton('Save', self)
        btnsave.setFont(font)
        btnsave.clicked.connect(self.button_click_save)
        
        btncanc = QtWidgets.QPushButton('Close', self)
        btncanc.setFont(font)
        btncanc.clicked.connect(self.button_click_cancel)

        hbox = QtWidgets.QHBoxLayout()
        hbox.addWidget(btnsave)                
        hbox.addWidget(btncanc)        

        vbox.addLayout(hbox)

        self.setLayout(vbox)  
        self.setWindowTitle(source)
        self.setWindowModality(QtCore.Qt.ApplicationModal)

    def button_click_cancel(self):
        self.close()

    def button_click_save(self):
        self.output = self.textfield.toPlainText()
        self.close()
   
class COBSComp(QtWidgets.QDialog):
    def __init__(self, destig, output, parent = None):
        super(COBSComp, self).__init__()
        vbox = QtWidgets.QVBoxLayout()
        
        elements = koprhand.loadcometid(destig)
        myobs = koprclass.icqcode()
        myobs.decode_icq(output)
        mydate = koprhand.timecvt(myobs.date)
        beg = mydate - datetime.timedelta(days = 30)
        end = mydate + datetime.timedelta(days = 30)
        filtobs = koprhand.loadcobscomet(destig, str(beg.strftime('%Y/%m/%d')), str(end.strftime('%Y/%m/%d')))
        xpt, ypt, h0, n = koprfunc.CompareObs(output,filtobs,destig)

        plots = [xpt, ypt, 'H0={:4.2f} n={:4.2f}'.format(h0,n), beg, end]

        self.sc = koprfig.CompCanavas(self, plots=plots )
        vbox.addWidget(self.sc)

        btncanc = QtWidgets.QPushButton('Close', self)
        btncanc.setFont(font)
        btncanc.clicked.connect(self.button_click_cancel)

        hbox = QtWidgets.QHBoxLayout()
        hbox.addWidget(btncanc)

        vbox.addLayout(hbox)
        
        self.setLayout(vbox)  
        self.setWindowTitle('COBS comparison')
        self.setWindowModality(QtCore.Qt.ApplicationModal)

    def button_click_cancel(self):
        self.close()

class AddDialog(QtWidgets.QDialog):
    def __init__(self, typ, parent = None):
        super(AddDialog, self).__init__()
        
        grid = QtWidgets.QGridLayout()
        if typ == 'loc':
            self.input, names, title, num, func = {}, [ 'Location name: ', 'Longitude: ', 'Latitude: ', 'Altitude(m): ', 'Timezone (GMT+): ' ],\
                                                  'Add New Location', 5, self.button_click_addloc
        elif typ == 'tel':
            self.input, names, title, num, func = {}, [ 'Telescope name: ', 'Type: ', 'Focal length(mm): ', 'Apertude diam.(cm): ' ],\
                                                  'Add New Telescope', 4, self.button_click_addtel
        elif typ == 'eye':
            self.input, names, title, num, func = {}, [ 'Eyepiece name: ', 'Focal length(mm): ', 'Apparent FOV: ' ],\
                                                  'Add New Eyepiece', 3, self.button_click_addeye
        elif typ == 'cam':
            self.input, names, title, num, func = {}, [ 'CCD Camera: ', 'Camera name:' ,'Camera code: ', 'Camera chip: ', 'Chip name: ',\
                                                  'Chip code: ', 'Resolution: (XxY px): ', 'Pixel size: (XxY µm): ' ], 'Add New Camera', 8, self.button_click_addcam

        for i in range(num):
            if i == 2 and typ == 'tel':
                self.lbl = QtWidgets.QLabel(names[i])
                self.lbl.setFont(font)
                grid.addWidget(self.lbl, *(i,0))
            else:
                label = QtWidgets.QLabel(names[i])
                label.setFont(font)
                grid.addWidget(label, *(i,0))

            if i == 1 and typ == 'tel':
                self.input[i] = QtWidgets.QComboBox(self)
                self.input[i].setStyleSheet("selection-color: black")
                self.input[i].setFixedWidth(150)
                for telescope in datavar.Telescopes():
                    self.input[i].addItem(telescope)
                self.input[i].activated[str].connect(self.setTelType)
            elif i == 0 and typ == 'cam':
                self.input[i] = QtWidgets.QComboBox(self)
                self.input[i].setStyleSheet("selection-color: black")
                self.input[i].setFixedWidth(150)
                for camera in datavar.Cameras():
                    self.input[i].addItem(camera)
                self.input[i].activated[str].connect(self.setCam)
            elif i == 3 and typ == 'cam':
                self.input[i] = QtWidgets.QComboBox(self)
                self.input[i].setStyleSheet("selection-color: black")
                self.input[i].setFixedWidth(150)
                for ccdchip in datavar.CCDChips():
                    self.input[i].addItem(ccdchip)
                self.input[i].activated[str].connect(self.setChip)
            else:
                self.input[i] = QtWidgets.QLineEdit(self)

            self.input[i].setFont(font)
            grid.addWidget(self.input[i], *(i,1))
       
        btn = QtWidgets.QPushButton(title, self)
        btn.setFont(font)
        btn.clicked.connect(func)
        grid.addWidget(btn, *(i+1,0))

        btncanc = QtWidgets.QPushButton('Cancel', self)
        btncanc.setFont(font)
        btncanc.clicked.connect(self.button_click_cancel)
        grid.addWidget(btncanc, *(i+1,1))
        
        self.setLayout(grid)  
        self.setWindowTitle(title)
        self.setWindowModality(QtCore.Qt.ApplicationModal)
        self.exec_()

    def button_click_cancel(self):
        self.close()
            
    def button_click_addloc(self):
        nlist, names = [str(self.input[0].text()).replace(',',''), str(self.input[1].text()).replace(',',''), str(self.input[2].text()).replace(',',''), str(self.input[3].text()).replace(',',''), str(self.input[4].text()).replace(',','')], []
        loc_file = open('./config-files/location.dat', 'r')
        for line in loc_file.readlines():
            if ',' in line:
                names.append(line.split(',')[0])
        loc_file.close()
        
        if '' in nlist:
            InfoDialog(1, 'Failed', 'Fill all required fields!')
        elif nlist[0] in names:
            InfoDialog(1, 'Failed', 'Name already exist!')
        else:
            nlocname, nlon, nlat, nelev, ngmt = nlist  
            if (':' in nlon) and (len(nlon.split(':')) == 3):
                nlon = str(int(nlon.split(':')[0]) + float(nlon.split(':')[1])/60. + float(nlon.split(':')[2])/3600.)
            elif (':' in nlon) and (len(nlon.split(':')) == 2):
                nlon = str(int(nlon.split(':')[0]) + float(nlon.split(':')[1])/60.)

            if (':' in nlat) and (len(nlat.split(':')) == 3):
                nlat = str(int(nlat.split(':')[0]) + float(nlat.split(':')[1])/60. + float(nlat.split(':')[2])/3600.)
            elif (':' in nlat) and (len(nlat.split(':')) == 2):
                nlat = str(int(nlat.split(':')[0]) + float(nlat.split(':')[1])/60.)
          
            loc_file = open('./config-files/location.dat', 'a')
            loc_file.write(nlocname +','+ nlon +','+ nlat +','+ nelev +','+ ngmt+'\n')
            loc_file.close()

            InfoDialog(0, 'Success', 'Location '+nlocname+' was added')
            
            self.close()

    def button_click_addtel(self):
        nlist, names = [str(self.input[0].text()), str(self.input[2].text()), str(self.input[3].text()), str(self.input[1].currentText()).split(':')[1].replace('\n','').replace(')','')], []
        tel_file = open('./config-files/telescope.dat', 'r')
        for line in tel_file.readlines():
            if ',' in line:
                names.append(line.split(',')[0])
        tel_file.close()
        
        if self.input[0].text() == '' or self.input[2].text() == '' or self.input[3].text() == '':
            InfoDialog(1, 'Failed', 'Fill all required fields!')
        elif nlist[0] in names:
            InfoDialog(1, 'Failed', 'Name already exist!')
        else:
            ntname, ntfocl, nltaper ,nttype = nlist
            nltaper = float(nltaper)*10
            tel_file = open('./config-files/telescope.dat', 'a')
            tel_file.write(ntname+','+nttype+','+str(int(nltaper))+','+ntfocl+'\n')
            tel_file.close()

            InfoDialog(0, 'Success', 'Telescope '+ntname+' was added')
            
            self.close()

    def setTelType(self,value):
        nttype = str(self.input[1].currentText()).split(':')[1].replace('\n','').replace(')','')
        if nttype == 'B':
            self.lbl.setText('Zoom(x): ')
        else:
            self.lbl.setText('Focal length(mm): ')
           
    def button_click_addeye(self):
        nlist, names = [str(self.input[0].text()).replace(',',''), str(self.input[1].text()).replace(',',''), str(self.input[2].text()).replace(',','')], []
        eye_file = open('./config-files/eyepiece.dat', 'r')
        for line in eye_file.readlines():
            if ',' in line:
                names.append(line.split(',')[0])
        eye_file.close()
        
        if '' in nlist:
            InfoDialog(1, 'Failed', 'Fill all required fields!')
        elif nlist[0] in names:
            InfoDialog(1, 'Failed', 'Name already exist!')
        else:
            nename, nefocl, nefov = nlist 
            eye_file = open('./config-files/eyepiece.dat', 'a')
            eye_file.write(nename+','+nefocl+','+nefov+'\n')
            eye_file.close()
    
            InfoDialog(0, 'Success', 'Eyepiece '+nename+' was added')

            self.close()

    def button_click_addcam(self):
        nlist, names = [str(self.input[1].text()).replace(',',''), str(self.input[2].text()).replace(',',''), str(self.input[4].text()).replace(',',''),\
                 str(self.input[5].text()).replace(',',''), str(self.input[6].text()).replace(',',''), str(self.input[7].text()).replace(',','')], []
        ccd_file = open('./config-files/camera.dat', 'r')
        for line in ccd_file.readlines():
            if ',' in line:
                names.append(line.split(',')[0])
        ccd_file.close()
        
        if '' in nlist:
            InfoDialog(1, 'Failed', 'Fill all required fields!')
        elif ('x' not in nlist[4]) or ('x' not in nlist[5]):
            InfoDialog(1, 'Failed', 'Missing x sign!')
        elif nlist[0] in names:
            InfoDialog(1, 'Failed', 'Name already exist!')
        else:
            nccdcam, nccdsc, nchip, nchipsc = nlist[0], nlist[1], nlist[2], nlist[3] 
            nxres, nyres = nlist[4].split('x')
            nxpix, nypix = nlist[5].split('x') 
            ccd_file = open('./config-files/camera.dat', 'a')
            ccd_file.write(nccdcam+','+nccdsc+','+nchip+','+nchipsc+','+nxres+','+nyres+','+nxpix+','+nypix+'\n')
            ccd_file.close()
    
            InfoDialog(0, 'Success', 'Camera '+nccdcam+' was added')
            
            self.close()

    def setCam(self,value):
        rest , nccdsc = value.split(':')
        nccdsc = nccdsc.replace('\n','').replace(')','').replace(' ','')
        nccdcam , rest = rest.split(' (')
        self.input[1].setText(nccdcam)
        self.input[2].setText(nccdsc)
                        
    def setChip(self,value):
        rest , nnchipsc = value.split(':')
        nnchipsc = nnchipsc.replace('\n','').replace(')','').replace(' ','')
        nchip , rest = rest.split(' (')
        self.input[4].setText(nchip)
        self.input[5].setText(nnchipsc)

# ************** DSLR edit ***************************
class ImgWiz(QtWidgets.QDialog):
    def __init__(self, parent = None):
        super(ImgWiz, self).__init__()

        hbox = QtWidgets.QHBoxLayout()
        self.listRaw = QtWidgets.QListWidget()
        self.listRaw.setFont(font)
        self.listRaw.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        
        self.listGFits = QtWidgets.QListWidget()
        self.listGFits.setFont(font)
        self.listGFits.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)

        vbox = QtWidgets.QVBoxLayout()
        label = QtWidgets.QLabel('Image list')
        label.setFont(font)
        vbox.addWidget(label)
        vbox.addWidget(self.listRaw)
        self.btnadd = QtWidgets.QPushButton('Open images', self)
        self.btnadd.setFont(font)
        self.btnadd.clicked.connect(self.button_click_add)
        vbox.addWidget(self.btnadd)
        self.btndark = QtWidgets.QPushButton('Dark field', self)
        self.btndark.setFont(font)
        self.btndark.clicked.connect(self.button_click_dark)
        vbox.addWidget(self.btndark)
        self.btnflat = QtWidgets.QPushButton('Flat field', self)
        self.btnflat.setFont(font)
        self.btnflat.clicked.connect(self.button_click_flat)
        vbox.addWidget(self.btnflat)
        hbox.addLayout(vbox)

        vbox = QtWidgets.QVBoxLayout()
        label = QtWidgets.QLabel('Images done')
        label.setFont(font)
        vbox.addWidget(label)
        vbox.addWidget(self.listGFits)
        self.btnclose = QtWidgets.QPushButton('Quit', self)
        self.btnclose.setFont(font)
        self.btnclose.clicked.connect(self.button_click_cancel)
        vbox.addWidget(self.btnclose)
        hbox.addLayout(vbox)

        self.setLayout(hbox)  
        self.setWindowTitle('DLSR Image tool')
        self.setWindowModality(QtCore.Qt.ApplicationModal)
        self.exec_()

    def button_click_cancel(self):
        self.close()

    def button_click_add(self):
        koprhand.MultipleImageLoad()

    def button_click_dark(self):
        self.close()

    def button_click_flat(self):
        self.close()

# ************** END ***************************
