#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import math
import koprdial
import koprobs
import datavar
import setupobs
from PyQt5 import QtCore, QtGui, QtWidgets
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar

font = datavar.DefaultFont()

## *******************************************
############## WIDGET ################
## *******************************************
class Comparison(QtWidgets.QWidget):
    
    def __init__(self, parent=None):
        super(Comparison, self).__init__()
        self.initUI()
        
    def initUI(self):
#                        unc  neg  bor  bval  coef pha  fsca fval bortle
#                         0    1    2    3     4    5    6    7    8               
        self.settings = [ 0,   0,   0,   1.5,  0,   0 ,  0,   1.,  0 ] # default settings
        self.script = []
        self.obs_list = [] # observations array
        self.types =    [['ccd', 1, 'o,circle'], ['vis', 1, 'x,cross'], ['mpct', 1, 'v,triangle_down'], ['mpcn', 1, '^,triangle_up']]
        self.obscode = [] # list of observers containing [ obscode, on/off ]

        # Input grid
        grid = QtWidgets.QGridLayout()
        
        btnnew = QtWidgets.QPushButton('Comparison script', self) 
        btnnew.clicked.connect(self.button_click_clear)

        btnset = QtWidgets.QPushButton('Settings and selections', self)
        btnset.clicked.connect(self.button_click_setup)
        
        elements = [ btnnew, btnset ]
        for i in range(2):
            elements[i].setFont(font)
            elements[i].setSizePolicy(QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum))
            grid.addWidget(elements[i], i, 0, 1, 2 )
            
        label = QtWidgets.QLabel('Regular formulas')
        label.setFont(font)
        grid.addWidget(label, 5, 0 )

        self.reg = QtWidgets.QTextEdit(self)
        self.reg.setFont(font) 
        self.reg.setMaximumHeight(btnnew.sizeHint().height() * 10)
        grid.addWidget(self.reg, 6, 0, 5, 2 )

        vboxl = QtWidgets.QVBoxLayout()
        vboxl.addLayout(grid)
        vboxl.addStretch(1)

        plotgrid = QtWidgets.QGridLayout()
        self.plotopt = [ [ QtWidgets.QLabel('Axis x ranges'), QtWidgets.QLineEdit(self), QtWidgets.QLineEdit(self), QtWidgets.QComboBox(self), QtWidgets.QPushButton('Default', self) ],
                         [ QtWidgets.QLabel('Axis y ranges'), QtWidgets.QLineEdit(self), QtWidgets.QLineEdit(self), QtWidgets.QComboBox(self), QtWidgets.QPushButton('Refresh', self) ] ]
        
        for i, line in enumerate(self.plotopt):
          for j, item in enumerate(self.plotopt[i]):
            self.plotopt[i][j].setFont(font)
            if j == 3:
              self.plotopt[i][j].setStyleSheet("selection-color: black")
            plotgrid.addWidget(self.plotopt[i][j], i, j) 

        for i, item in enumerate( ['Date','Days from T','r','Log(r)'] ):
          self.plotopt[0][3].addItem(item)
        self.plotopt[0][3].activated[str].connect(self.onActivatedR)
        self.dater = self.plotopt[0][3].currentIndex() 

        
        for i, item in enumerate( ['mag','mag - 5*log(dist)'] ):
          self.plotopt[1][3].addItem(item)
        self.plotopt[1][3].activated[str].connect(self.onActivatedR)
        
        self.plotopt[0][4].clicked.connect(self.button_click_default)
        self.plotopt[1][4].clicked.connect(self.button_click_update)

        self.plotopt[0][3].setCurrentIndex(3)
        self.plotopt[1][3].setCurrentIndex(1)
        
        self.sc = koprobs.LightCurve(self)
        self.sc.setaxes2()
        self.mpl_toolbar = NavigationToolbar(self.sc.canvas, self)

        vboxr = QtWidgets.QVBoxLayout()
        hbox = QtWidgets.QHBoxLayout()
        hbox.addLayout(plotgrid)
        hbox.addStretch(1)
        vboxr.addLayout(hbox)
        vboxr.addWidget(self.mpl_toolbar)
        vboxr.addWidget(self.sc)

        hbox = QtWidgets.QHBoxLayout()
        hbox.addLayout(vboxl)
        hbox.addLayout(vboxr) 
        
        self.setLayout(hbox)            
        self.setWindowTitle('KOPR - Comparison')    
        self.show()               

### ************ NEW FUNCTIONS *********************
    def onActivatedR(self):
        if self.obs_list:         
          self.DoPlotUpdate()

    def button_click_default(self):
        self.obs_list, self.obscode, text = koprobs.LoadMultipleData(self.script, self.settings[7]) #load data
        self.reg.setText('\n'.join(text))
        self.DoPlotUpdate()

    def button_click_update(self):
        self.DoPlotUpdate( self.GetLimits() )

    def button_click_clear(self):
        self.datarang = []
        self.UpdRangeVal() 
        
        a = koprdial.CometSelector( self.script )
        a.exec_()    
        self.script = a.output
        
        if self.script != '':
          try:
            self.obs_list, self.obscode, text = koprobs.LoadMultipleData(self.script, self.settings[7]) #load data
            self.reg.setText('\n'.join(text))
            self.DoPlotUpdate()
          except:
            koprdial.InfoDialog(1, 'Script error', 'Correct your script.')

    def DoPlotUpdate(self, lim = [], typ = 0, obs = 0 ):
        rangtype = self.plotopt[0][3].currentIndex()
        if rangtype < 2:
          self.sc.fig.clf()
          self.sc.setaxes1()
        else:
          self.sc.fig.clf()
          self.sc.setaxes2()
        
        xyranges, legend = [ [], [], [], [] ], []
        i, j, k, l = 8 + rangtype, self.plotopt[1][3].currentIndex(),  self.settings[5], self.settings[6] # type, dist corr, phase corr, forwsca corr
        for line in self.obs_list: # line for each comet
          for data in line[3]: # observations for each return
            for types in self.types:
              datat = [x for x in data[2] if x[0] == types[0] ]
              if datat != []:
                xyr = self.PlotPoints(datat, lim, i, j, k, l, types[2].split(',')[0], line[0], typ, obs)
                # add to ranges
                if len(xyr) == 4:
                  xyranges[0].append(xyr[0])
                  xyranges[1].append(xyr[1])
                  xyranges[2].append(xyr[2])
                  xyranges[3].append(xyr[3])
          legend.append(mlines.Line2D([], [], color=line[0], marker=types[2].split(',')[0], label=line[1]))
        xyranges = self.DelNA(xyranges)

        # Plot lines
        if rangtype == 0:
          limits = [ min(xyranges[0]), max(xyranges[1]), min(xyranges[2]), max(xyranges[3]) ]
        else:
          limits = [ round(min(xyranges[0]), 1), round(max(xyranges[1]), 1), round( min(xyranges[2]), 1 ), round( max(xyranges[3]), 1 ) ]
        set_filter = [rangtype, limits[0], limits[1] ]

        for line in self.obs_list: # line for each comet
          mp_arrray = koprobs.magparams(line[2])          
          xyr = self.PlotLines(mp_arrray, set_filter, line, j, k, l)
          # add to ranges
          if len(xyr) == 4:
            xyranges[0].append(xyr[0])
            xyranges[1].append(xyr[1])
            xyranges[2].append(xyr[2])
            xyranges[3].append(xyr[3])
        xyranges = self.DelNA(xyranges)

        if len(lim) == 0:
          self.SetRang( min(xyranges[0]), max(xyranges[1]), min(xyranges[2]), max(xyranges[3]), 0 )
        
          if self.plotopt[0][3].currentIndex() == 0:
            self.UpdRangeVal( [ koprobs.DtimeToStr(self.datarang[0]), 
                                koprobs.DtimeToStr(self.datarang[1]), 
                                str( round(self.datarang[3], 1)), 
                                str( round(self.datarang[2], 1)) ] )
          else:
            self.UpdRangeVal( [ str(round(self.datarang[0], 1)), 
                                str(round(self.datarang[1], 1)), 
                                str(round(self.datarang[3], 1)), 
                                str(round(self.datarang[2], 1)) ] )

        rng = datavar.RangeMargins( rangtype )

        if rangtype < 2:
          self.sc.ax.legend(handles=legend, fontsize = 8)
          if len(lim) == 0:
            self.sc.ax.set_xlim( ( self.datarang[0] - rng, self.datarang[1] + rng) )
            self.sc.ax.set_ylim( ( self.datarang[3] +1., self.datarang[2] -1.) )
          elif len(lim) == 4:
            self.sc.ax.set_xlim( ( lim[0], lim[1] ) )
            self.sc.ax.set_ylim( ( lim[2], lim[3] ) )
          self.sc.ax.tick_params(axis='both', which='major', labelsize=8)     
        else:        
          self.sc.ax2.legend(handles=legend, fontsize = 8)
          if len(lim) == 0:
            self.sc.ax1.set_xlim( ( self.datarang[0] - rng, self.datarang[1] + rng) )
            self.sc.ax1.set_ylim( ( self.datarang[3] +1., self.datarang[2] -1.) )
            self.sc.ax2.set_xlim( ( self.datarang[0] - rng, self.datarang[1] + rng) )
            self.sc.ax2.set_ylim( ( self.datarang[3] +1., self.datarang[2] -1.) )
          elif len(lim) == 4:
            self.sc.ax1.set_xlim( ( lim[0], lim[1]) )
            self.sc.ax1.set_ylim( ( lim[2], lim[3]) )
            self.sc.ax2.set_xlim( ( lim[0], lim[1]) )
            self.sc.ax2.set_ylim( ( lim[2], lim[3]) )
          self.sc.ax1.invert_xaxis()          
          self.sc.ax1.tick_params(axis='both', which='major', labelsize=8)     
          self.sc.ax2.tick_params(axis='both', which='major', labelsize=8)     

        self.sc.fig.suptitle('Light curve comparison')
        self.sc.canvas.draw()   

    def DelNA(self, xyranges):# remove n/a values
        for idx in range(4):
          xyranges[idx] = [x for x in xyranges[idx] if x != 'n/a']
        return xyranges

    def MinMax(self, xp, yp):      
        if len(xp) > 0 : 
          minx, maxx = min(xp), max(xp)
        else:
          minx, maxx = 'n/a', 'n/a'
        
        if len(yp) > 0 : 
          miny, maxy = min(yp), max(yp)
        else:
          miny, maxy = 'n/a', 'n/a'
          
        return minx, maxx, miny, maxy     

    def PlotPoints(self, data, lim, i, j, k, l, m, c, typ, obs):
        # remove negative or uncertain
        if self.settings[0] == 1:
          data = [x for x in data if x[2] != ':']
        if self.settings[1] == 1:
          data = [x for x in data if x[3] != '!']
        
        xpoints1, ypoints1, xpoints2, ypoints2 = koprobs.xypoints(data, lim, i, j, k, l, typ, obs)

        if self.plotopt[0][3].currentIndex() < 2:
          self.sc.ax.plot( xpoints1 + xpoints2, ypoints1 + ypoints2, marker=m, color=c, markersize = 3, ls = 'None' )
        else:
          self.sc.ax1.plot( xpoints1, ypoints1, marker=m, color=c, markersize = 3, ls = 'None' )
          self.sc.ax2.plot( xpoints2, ypoints2, marker=m, color=c, markersize = 3, ls = 'None' )
          
        return self.MinMax(xpoints1+xpoints2, ypoints1+ypoints2) 

    def PlotLines(self, mp_arrray, set_filter, line, j, k, l):
        i = set_filter[0]
        j = 1 - j
        for data in line[3]:
          eline = [x for x in list(data[1].values()) if x[0] == data[0]][0][1]
          halfw = koprobs.HalfW(eline)

          for param in mp_arrray:
            param = koprobs.halfwnorm(param, halfw)
            points = koprobs.GeneratePoints(eline, param, set_filter, self.settings[7])
            xpoints1, xpoints2 = [ x[i+4] for x in points if x[5] < 0], [x[i+4] for x in points if x[5] >= 0]
            ypoints1, ypoints2 = [ x[0] + j*x[1] + k*x[2] + l*x[3] for x in points if x[5] < 0], [ x[0] + j*x[1] + k*x[2] + l*x[3] for x in points if x[5] >= 0]
            if i < 2:
              self.sc.ax.plot( xpoints1 + xpoints2, ypoints1 + ypoints2, color=line[0], lw = 2, aa = True )
            else:
              self.sc.ax1.plot( xpoints1, ypoints1, color=line[0], lw = 2, aa = True )
              self.sc.ax2.plot( xpoints2, ypoints2, color=line[0], lw = 2, aa = True )            

        return self.MinMax(xpoints1+xpoints2, ypoints1+ypoints2) 

    def SetRang(self, xmin, xmax, ymin, ymax, user ):
        if user == 0: # rounding values
          if self.plotopt[0][3].currentIndex() == 0: # datum
            xmin, xmax = koprobs.RoundDates( xmin, xmax )
          elif self.plotopt[0][3].currentIndex() == 1:
            xmin, xmax = math.floor(xmin), math.ceil(xmax)
          elif self.plotopt[0][3].currentIndex() == 2:
            xmin, xmax = math.floor(xmin*10.)/10., math.ceil(xmax*10.)/10.
          elif self.plotopt[0][3].currentIndex() == 3:
            xmin, xmax = math.floor(xmin*100.)/100., math.ceil(xmax*100.)/100.
          ymin, ymax = math.floor(ymin*10.)/10., math.ceil(ymax*10.)/10.
        
        self.datarang = [ xmin, xmax, ymin, ymax ]
    
    def button_click_setup(self):
        tabdialog = setupobs.TabDialogComp(self.settings, self.types, self.obscode)
        tabdialog.exec_()
        settings = tabdialog.settings
        types = tabdialog.cat
        obscode = tabdialog.obscode
        if len( [x for x in self.types if x[1] == 1 ] ) != len( [x for x in types if x[1] == 1 ] ):
          self.types = types
          self.DoPlotUpdate(typ = 1)

        if len( [x for x in self.obscode if x[1] == 1 ] ) != len( [x for x in obscode if x[1] == 1 ] ):
          self.obscode = obscode
          self.DoPlotUpdate(obs = 1)
        
        if settings != self.settings:  
          self.settings = settings
          self.DoPlotUpdate()        

    def GetLimits(self):
        if self.plotopt[0][3].currentIndex() == 0:
          return [ koprobs.StrToDtime( self.plotopt[0][1].text() ) , koprobs.StrToDtime( self.plotopt[0][2].text()) , float(self.plotopt[1][1].text()) , float(self.plotopt[1][2].text()) ]
        else:
          return [ float( self.plotopt[0][1].text() ) , float( self.plotopt[0][2].text()) , float(self.plotopt[1][1].text()) , float(self.plotopt[1][2].text()) ]

    def UpdRangeVal(self, text = [ '','','','' ]):
        self.plotopt[0][1].setText(text[0])
        self.plotopt[0][2].setText(text[1])
        self.plotopt[1][1].setText(text[2])
        self.plotopt[1][2].setText(text[3])

### ************ END NEW FUNCTIONS ********************* 
                
class ScrollComparison(QtWidgets.QScrollArea):
    """ This scroll area only handles one widget for which scrolling should be provided. """
    def __init__(self, parent=None):
        QtWidgets.QScrollArea.__init__(self, parent)

        self.w = Comparison(self)
        self.setWidget(self.w)    
